const influx = require("influx");

const inflog = {

    /**
     * inflog.createInstance
     * @returns {Promise}
     */
    createInstance() {

        return new Promise((resolve, reject) => {

            const influxObject = inflog.getInstance();

            influxObject.getDatabaseNames()

                .then(async (nameObjects) => {

                    if (nameObjects.includes(process.env.INFLOG_DATABASE) === false) {

                        await influxObject.createDatabase(process.env.INFLOG_DATABASE);

                        resolve(influxObject);

                    }

                    resolve(influxObject);

                })

                .catch((error) => {

                    reject(error);

                });

        });

    },

    /**
     * inflog.createRecord
     * @param {int} level
     * @param {string} message
     * @returns {boolean}
     */
    createRecord(level, message) {

        // Step #1
        // Make sure the INFLOG is enabled, if not, just print it out.

        if (process.env.INFLOG !== "true") {

            console.log(message);

            return false;

        }

        // Step #2
        // Make sure the "level" is greater than "debug", if not, just print it out.

        if (level !== 0 && level !== 1 && level !== 2) {

            console.log(message);

            return false;

        }

        // Step #3
        // Write this log into InfluxDB

        const influxObject = inflog.getInstance();

        influxObject.writePoints([{
            measurement: process.env.INFLOG_MEASUREMENT,
            tags: {
                instance: process.env.INFLOG_INSTANCE
            },
            fields: {
                level: level,
                message: message
            },
        }]);

        return true;

    },

    /**
     * inflog.getInstance
     * @returns {InfluxDB}
     */
    getInstance() {

        return new influx.InfluxDB({
            host: process.env.INFLOG_HOST,
            port: process.env.INFLOG_PORT,
            database: process.env.INFLOG_DATABASE,
            schema: [
                {
                    measurement: process.env.INFLOG_MEASUREMENT,
                    tags: ["instance"],
                    fields: {
                        level: influx.FieldType.INTEGER,
                        message: influx.FieldType.STRING
                    }
                }
            ]
        });

    },

    /**
     * inflog.error
     * @param {string} message
     */
    error(message) {
        inflog.createRecord(0, message);
    },

    /**
     * inflog.warning
     * @param {string} message
     */
    warning(message) {
        inflog.createRecord(1, message);
    },

    /**
     * inflog.info
     * @param {string} message
     */
    info(message) {
        inflog.createRecord(2, message);
    },

    /**
     * inflog.verbose
     * @param {string} message
     */
    verbose(message) {
        inflog.createRecord(3, message);
    },

    /**
     * inflog.debug
     * @param {string} message
     */
    debug(message) {
        inflog.createRecord(4, message);
    }

};

module.exports = inflog;